@@ -16,6 +16,8 @@ from kensu.utils.kensu_provider import KensuProvider
 import google.cloud.bigquery as bq
 import google.cloud.bigquery.job as bqj
 
+logger = logging.getLogger(__name__)
+
 
 class QueryJob(bqj.QueryJob):
     # - client.query(sql) returns QueryJob which we patch here for tracking
@@ -51,20 +53,28 @@ class QueryJob(bqj.QueryJob):
         return job
 
     @staticmethod
-    def override_result(job: bqj.QueryJob) -> bqj.QueryJob:
+    def override_result(job #  type: google.cloud.bigquery.job.QueryJob
+                        ) -> bqj.QueryJob:
         f = job.result
 
         def wrapper(*args, **kwargs):
             kensu = KensuProvider().instance()
             result = f(*args, **kwargs)
             client = kensu.data_collectors['BigQuery']
-            dest = job.destination
+            ddl_target_table = job.ddl_target_table
+            dest = job.destination or ddl_target_table
+            is_ddl_write = bool(ddl_target_table)
+            logger.debug(f'in QueryJob.result(): dest={dest}')
+            logger.debug(f'in QueryJob.result(): referenced={job.referenced_tables}')
             if isinstance(dest, bq.TableReference):
                 dest = client.get_table(dest)
+
+            #TODO What if several SELECT queries linked with ; ?
             db_metadata, table_id_to_bqtable, table_infos = BqOfflineParser.get_referenced_tables_metadata(
                 kensu=kensu,
                 client=client,
-                query=job.query)
+                job=job,
+                query=BqOfflineParser.normalize_table_refs(job.query))
             try:
                 bq_lineage = BqRemoteParser.parse(
                     kensu=kensu,
@@ -73,7 +83,7 @@ class QueryJob(bqj.QueryJob):
                     db_metadata=db_metadata,
                     table_id_to_bqtable=table_id_to_bqtable)
             except:
-                logging.warning("Error in BigQuery collector, using fallback implementation")
+                logger.warning("Error in BigQuery collector, using fallback implementation")
                 traceback.print_exc()
                 bq_lineage = BqOfflineParser.fallback_lineage(kensu, table_infos, dest)
 
@@ -83,11 +93,15 @@ class QueryJob(bqj.QueryJob):
                 ksu=kensu,
                 df_result=result,
                 operation_type='BigQuery SQL result',
-                report_output=kensu.report_in_mem,
+                report_output=kensu.report_in_mem or is_ddl_write,
                 # FIXME: how to know when in mem or when bigquery://projects/psyched-freedom-306508/datasets/_b63f45da1cafbd073e5c2770447d963532ac43ec/tables/anonc79d9038a13ab2dbe40064636b0aceedc62b5d69
-                register_output_orig_data=False  # FIXME? when do we need this? INSERT INTO?
+                register_output_orig_data=is_ddl_write  # used for DDL writes, like CREATE TABLE t2 AS SELECT * FROM t1
+                # FIXME: how about INSERT INTO?
             )
-            # FIXME? kensu.report_with_mapping()
+            if is_ddl_write:
+                # FIXME: report_with_mapping fails with empty inputs/lineage?
+                if len(bq_lineage.lineage) > 0:
+                    kensu.report_with_mapping()
 
             return result
 
@@ -101,5 +115,5 @@ class QueryJob(bqj.QueryJob):
             if (dest is not None) and isinstance(dest, google.cloud.bigquery.table.Table):
                 result.ksu_dest = dest
         except:
-            logging.warning('setting _store_bigquery_job_destination failed')
+            logger.warning('setting _store_bigquery_job_destination failed')
             traceback.print_exc()
