@@ -8,11 +8,11 @@ class KensuDatasourceAndSchema:
                  ksu_ds,
                  ksu_schema,
                  f_get_stats=lambda: None,
-                 f_publish_stats = lambda lineage_run_id : {}):
+                 f_publish_stats=lambda lineage_run_id: {}):
         """
         :param kensu.client.DataSource ksu_ds: Datasource info which was extracted from external system
         :param kensu.client.Schema ksu_schema: Schema info for the datasource
-        :param () -> typing.Dict[string, float] f_get_stats: [Optional] a function to retrieve DataStats for this DataSource
+        :param () -> typing.Union[typing.Dict[string, float], None] f_get_stats: [Optional] a function to retrieve DataStats for this DataSource
         :param (str) -> () f_publish_stats : [Optional] a function publishing the statistics given a lineage run guid
         """
 
@@ -67,6 +67,12 @@ class ExtDependencyEntry:
         return 'ExtDependencyEntry(input_ds={}, lineage={})'.format(str(self.input_ds), str(self.lineage))
 
 
+def filter_matching_fields(fields, other_fields):
+    """return fields which are same as in other_fields list, ignoring the case"""
+    other_fields_lowercase = set([f.lower() for f in other_fields])
+    return [f for f in fields if f.lower() in other_fields_lowercase]
+
+
 class GenericComputedInMemDs:
     def __init__(
             self,
@@ -81,6 +87,44 @@ class GenericComputedInMemDs:
     def __repr__(self):
         return 'GenericComputedInMemDs(lineage={})'.format(str(self.lineage))
 
+    @staticmethod
+    def for_direct_or_full_mapping(
+            all_inputs,  # type: list[KensuDatasourceAndSchema]
+            out_field_names # type: list[str]
+                                   ):
+        inputs_with_matching_fields = [(i, filter_matching_fields(i.field_names(), out_field_names))
+                                       for i in all_inputs
+                                       if len(filter_matching_fields(i.field_names(), out_field_names)) > 0]
+
+        global_lineage = []
+
+        if len(inputs_with_matching_fields) > 0:
+            # direct lineage only for matching fields/DSes
+            for input, matching_input_fields in inputs_with_matching_fields:
+                matching_output_fields = filter_matching_fields(out_field_names, matching_input_fields)
+                lin_entry = ExtDependencyEntry(
+                    input_ds=input,
+                    # lineage: dict[str-output field, list[str]-input fields]
+                    lineage=dict(
+                        [(out_field_name, filter_matching_fields(input.field_names(), [out_field_name]))
+                         for out_field_name in matching_output_fields])
+                )
+                global_lineage.append(lin_entry)
+        else:
+            # all-to-all lineage between all output fields & all input DSes fields
+            for input in all_inputs:
+                lin_entry = ExtDependencyEntry(
+                    input_ds=input,
+                    # lineage: dict[str-output field, list[str]-input fields]
+                    lineage=dict(
+                        [(out_field_name, [v.name for v in input.field_names()])
+                         for out_field_name in out_field_names])
+                )
+                global_lineage.append(lin_entry)
+        return GenericComputedInMemDs(lineage=global_lineage)
+
+
+
     # FIXME: rename register_output_orig_data -> output_is_not_in_mem ?
     def report(self, ksu, df_result, operation_type, report_output=False, register_output_orig_data=False):
         from kensu.utils.helpers import extract_ksu_ds_schema
